@@ -1185,6 +1185,10 @@ class Node:
             create_err=True,
         )
 
+        self.resource_isolation_config.add_system_pids(
+            self._get_system_processes_for_resource_isolation()
+        )
+
         process_info = ray._private.services.start_raylet(
             self.redis_address,
             self.gcs_address,
@@ -1422,10 +1426,20 @@ class Node:
         if self.resource_isolation_config.is_enabled():
             self.resource_isolation_config.add_object_store_memory(object_store_memory)
 
-        self.start_raylet(plasma_directory, fallback_directory, object_store_memory)
         if self._ray_params.include_log_monitor:
             self.start_log_monitor()
 
+        self.start_raylet(plasma_directory, fallback_directory, object_store_memory)
+
+    def _get_system_processes_for_resource_isolation(self) -> str:
+        """Returns a list of system processes that will be isolated by raylet.
+
+        NOTE: If a new system process is started before the raylet starts up, it needs to be
+        added to self.all_processes so it can be moved into the raylet's managed cgroup
+        hierarchy.
+        """
+        return ",".join(str(p[0].process.pid) for p in self.all_processes.values())
+
     def _kill_process_type(
         self,
         process_type,
